package me.senseiwells.essentialclient.feature.chunkdebug;

import com.mojang.blaze3d.systems.RenderSystem;
import me.senseiwells.essentialclient.EssentialClient;
import me.senseiwells.essentialclient.feature.keybinds.ClientKeyBinds;
import me.senseiwells.essentialclient.utils.EssentialUtils;
import me.senseiwells.essentialclient.utils.render.ChildScreen;
import me.senseiwells.essentialclient.utils.render.Texts;
import net.minecraft.client.MinecraftClient;
import net.minecraft.client.font.TextRenderer;
import net.minecraft.client.gui.DrawableHelper;
import net.minecraft.client.gui.screen.Screen;
import net.minecraft.client.gui.widget.ButtonWidget;
import net.minecraft.client.gui.widget.TextFieldWidget;
import net.minecraft.client.render.*;
import net.minecraft.client.util.math.MatrixStack;
import net.minecraft.text.Text;

public class ChunkDebugScreen extends ChildScreen {
	public static final int
		HEADER_HEIGHT = 20,
		FOOTER_ROW_HEIGHT = 20,
		FOOTER_ROW_PADDING = 5,
		FOOTER_ROW_COUNT = 2,
		FOOTER_HEIGHT = FOOTER_ROW_HEIGHT * FOOTER_ROW_COUNT + FOOTER_ROW_PADDING * (FOOTER_ROW_COUNT + 1);

	private final MinecraftClient client = EssentialUtils.getClient();
	private NumberFieldWidget xPositionBox;
	private NumberFieldWidget zPositionBox;
	private boolean canClick = false;

	public ChunkDebugScreen(Screen parent) {
		super(Texts.CHUNK_SCREEN, parent);
	}

	@Override
	public void init() {
		if (ChunkGrid.instance == null) {
			ChunkGrid.instance = new ChunkGrid(this.client, this.width, this.height);
		}
		EssentialClient.CHUNK_NET_HANDLER.requestChunkData(ChunkGrid.instance.getDimension());
		int buttonWidth = (this.width - FOOTER_ROW_PADDING * 4) / 3;
		int buttonHeight = this.height - FOOTER_ROW_HEIGHT * 3 + FOOTER_ROW_PADDING * 2;
		Text dimensionText = Texts.literal(ChunkGrid.instance.getPrettyDimension());
		ButtonWidget dimensionButton = this.addDrawableChild(new ButtonWidget(FOOTER_ROW_PADDING, buttonHeight, buttonWidth, FOOTER_ROW_HEIGHT, dimensionText, button -> {
			ChunkGrid.instance.cycleDimension();
			button.setMessage(Texts.literal(ChunkGrid.instance.getPrettyDimension()));
			EssentialClient.CHUNK_NET_HANDLER.requestChunkData(ChunkGrid.instance.getDimension());
		}));
		this.addDrawableChild(new ButtonWidget(buttonWidth + FOOTER_ROW_PADDING * 2, buttonHeight, buttonWidth, FOOTER_ROW_HEIGHT, Texts.literal("Return to player"), button -> {
			if (this.client.player != null) {
				ChunkGrid.instance.setDimension(this.client.player.world);
				dimensionButton.setMessage(Texts.literal(ChunkGrid.instance.getPrettyDimension()));
				int chunkX = this.client.player.getChunkPos().x;
				int chunkZ = this.client.player.getChunkPos().z;
				ChunkGrid.instance.setCentre(chunkX, chunkZ);
				this.xPositionBox.setText(String.valueOf(chunkX));
				this.zPositionBox.setText(String.valueOf(chunkZ));
				EssentialClient.CHUNK_NET_HANDLER.requestChunkData(ChunkGrid.instance.getDimension());
			}
		}));
		Text initialMinimapText = Texts.literal("Minimap: %s".formatted(ChunkGrid.instance.getMinimapMode().prettyName));
		this.addDrawableChild(new ButtonWidget(buttonWidth * 2 + FOOTER_ROW_PADDING * 3, buttonHeight, buttonWidth, FOOTER_ROW_HEIGHT, initialMinimapText, button -> {
			ChunkGrid.instance.cycleMinimap();
			Text minimapText = Texts.literal("Minimap: %s".formatted(ChunkGrid.instance.getMinimapMode().prettyName));
			button.setMessage(minimapText);
		}));
		buttonHeight = this.height - FOOTER_ROW_HEIGHT * 2 + FOOTER_ROW_PADDING * 3;
		this.xPositionBox = new NumberFieldWidget(this.textRenderer, FOOTER_ROW_PADDING + 28, buttonHeight, buttonWidth - 30, 20, Texts.literal("X"));
		this.xPositionBox.setInitialValue(ChunkGrid.instance.getCentreX());
		this.zPositionBox = new NumberFieldWidget(this.textRenderer, buttonWidth + FOOTER_ROW_PADDING * 2 + 28, buttonHeight, buttonWidth - 30, 20, Texts.literal("Z"));
		this.zPositionBox.setInitialValue(ChunkGrid.instance.getCentreZ());
		this.addDrawableChild(new ButtonWidget(buttonWidth * 2 + FOOTER_ROW_PADDING * 3, buttonHeight, buttonWidth, FOOTER_ROW_HEIGHT, Texts.literal("Refresh"), button -> {
			if (hasControlDown()) {
				ChunkHandler.clearAllChunks();
				EssentialClient.CHUNK_NET_HANDLER.requestServerRefresh();
			}
			else {
				ChunkHandler.clearAllChunks();
				EssentialClient.CHUNK_NET_HANDLER.requestChunkData(ChunkGrid.instance.getDimension());
			}
		}));

		this.addDrawableChild(this.xPositionBox);
		this.addDrawableChild(this.zPositionBox);
	}

	@Override
	public void resize(MinecraftClient client, int width, int height) {
		if (ChunkGrid.instance != null) {
			ChunkGrid.instance.onResize(width, height - HEADER_HEIGHT - FOOTER_HEIGHT);
		}
		super.resize(client, width, height);
	}

	@Override
	public void tick() {
		super.tick();
		this.xPositionBox.tick();
		this.zPositionBox.tick();
	}

	@Override
	public void onClose() {
		if (ChunkGrid.instance.getMinimapMode() == ChunkGrid.Minimap.NONE) {
			EssentialClient.CHUNK_NET_HANDLER.requestChunkData();
			ChunkHandler.clearAllChunks();
		}
		super.onClose();
	}

	@Override
	public void render(MatrixStack matrices, int mouseX, int mouseY, float delta) {
		this.renderBackground(matrices);

		ChunkGrid.instance.render(0, HEADER_HEIGHT, this.width, this.height - HEADER_HEIGHT - FOOTER_HEIGHT, false);

		Tessellator tessellator = Tessellator.getInstance();
		BufferBuilder bufferBuilder = tessellator.getBuffer();

		this.drawHeaderAndFooterGradient(tessellator, bufferBuilder);
		this.drawHeaderAndFooterTexture(tessellator, bufferBuilder);

		if (ChunkGrid.instance.isPanning()) {
			this.xPositionBox.setText(String.valueOf(ChunkGrid.instance.getCentreX()));
			this.zPositionBox.setText(String.valueOf(ChunkGrid.instance.getCentreZ()));
		}

		this.drawText(matrices, "Chunk Debug Map", this.width / 2, 8, 1, true);
		if (ChunkGrid.instance.getSelectionText() != null) {
			this.drawText(matrices, ChunkGrid.instance.getSelectionText(), this.width / 2, 30, 1, true);
		}

		this.xPositionBox.render(matrices, mouseX, mouseY, delta);
		this.zPositionBox.render(matrices, mouseX, mouseY, delta);

		int textHeight = this.height - 20;
		int xOffset = FOOTER_ROW_PADDING + 10;
		int zOffset = this.xPositionBox.getWidth() + 50;

		this.drawText(matrices, "X", xOffset, textHeight, 1.5F, false);
		this.drawText(matrices, "Z", zOffset, textHeight, 1.5F, false);

		super.render(matrices, mouseX, mouseY, delta);
	}

	private void drawText(MatrixStack matrices, String text, int x, int y, float scale, boolean center) {
		matrices.push();
		matrices.translate(x, y, 0.0D);
		matrices.scale(scale, scale, 0.0F);
		if (center) {
			DrawableHelper.drawCenteredText(matrices, this.textRenderer, Texts.literal(text), 0, 0, 0xFFFFFF);
		}
		else {
			DrawableHelper.drawTextWithShadow(matrices, this.textRenderer, Texts.literal(text), 0, 0, 0xFFFFFF);
		}
		matrices.pop();
	}

	private void drawHeaderAndFooterGradient(Tessellator tessellator, BufferBuilder bufferBuilder) {
		RenderSystem.disableTexture();
		RenderSystem.enableBlend();
		RenderSystem.setShaderColor(1.0F, 1.0F, 1.0F, 1.0F);

		bufferBuilder.begin(VertexFormat.DrawMode.QUADS, VertexFormats.POSITION_COLOR);

		// Header gradient
		bufferBuilder.vertex(0, HEADER_HEIGHT, 0).color(0, 0, 0, 255).next();
		bufferBuilder.vertex(0, HEADER_HEIGHT + 4, 0).color(0, 0, 0, 0).next();
		bufferBuilder.vertex(this.width, HEADER_HEIGHT + 4, 0).color(0, 0, 0, 0).next();
		bufferBuilder.vertex(this.width, HEADER_HEIGHT, 0).color(0, 0, 0, 255).next();

		// Footer gradient
		bufferBuilder.vertex(0, this.height - FOOTER_HEIGHT - 4, 0).color(0, 0, 0, 0).next();
		bufferBuilder.vertex(0, this.height - FOOTER_HEIGHT, 0).color(0, 0, 0, 255).next();
		bufferBuilder.vertex(this.width, this.height - FOOTER_HEIGHT, 0).color(0, 0, 0, 255).next();
		bufferBuilder.vertex(this.width, this.height - FOOTER_HEIGHT - 4, 0).color(0, 0, 0, 0).next();

		tessellator.draw();
	}

	private void drawHeaderAndFooterTexture(Tessellator tessellator, BufferBuilder bufferBuilder) {
		RenderSystem.setShader(GameRenderer::getPositionTexColorShader);
		RenderSystem.setShaderTexture(0, OPTIONS_BACKGROUND_TEXTURE);
		RenderSystem.setShaderColor(1.0F, 1.0F, 1.0F, 1.0F);
		RenderSystem.enableTexture();
		RenderSystem.disableBlend();

		bufferBuilder.begin(VertexFormat.DrawMode.QUADS, VertexFormats.POSITION_TEXTURE_COLOR);

		// Header
		bufferBuilder.vertex(0, 0, 0).texture( 0, 0).color(64, 64, 64, 255).next();
		bufferBuilder.vertex(0, HEADER_HEIGHT, 0).texture( 0, HEADER_HEIGHT / 32f).color(64, 64, 64, 255).next();
		bufferBuilder.vertex(this.width, HEADER_HEIGHT, 0).texture(this.width / 32f, HEADER_HEIGHT / 32f).color(64, 64, 64, 255).next();
		bufferBuilder.vertex(this.width, 0, 0).texture(this.width / 32f, 0).color(64, 64, 64, 255).next();

		// Footer
		bufferBuilder.vertex(0, this.height - FOOTER_HEIGHT, 0).texture(0, (this.height - FOOTER_HEIGHT) / 32f).color(64, 64, 64, 255).next();
		bufferBuilder.vertex(0, this.height, 0).texture(0, this.height / 32f).color(64, 64, 64, 255).next();
		bufferBuilder.vertex(this.width, this.height, 0).texture(this.width / 32f, this.height / 32f).color(64, 64, 64, 255).next();
		bufferBuilder.vertex(this.width, this.height - FOOTER_HEIGHT, 0).texture(this.width / 32f, (this.height - FOOTER_HEIGHT) / 32f).color(64, 64, 64, 255).next();

		tessellator.draw();
	}

	@Override
	public boolean keyPressed(int keyCode, int scanCode, int modifiers) {
		if (ClientKeyBinds.OPEN_CHUNK_DEBUG.matchesKey(keyCode, scanCode)) {
			this.onClose();
			return true;
		}
		return super.keyPressed(keyCode, scanCode, modifiers);
	}

	@Override
	public boolean mouseScrolled(double mouseX, double mouseY, double amount) {
		return ChunkGrid.instance.onScroll(mouseX, mouseY, amount);
	}

	@Override
	public boolean mouseClicked(double mouseX, double mouseY, int button) {
		this.canClick = true;
		ChunkGrid.instance.onClicked(mouseX, mouseY, button);
		return super.mouseClicked(mouseX, mouseY, button);
	}

	@Override
	public boolean mouseReleased(double mouseX, double mouseY, int button) {
		if (this.canClick) {
			ChunkGrid.instance.onRelease(mouseX, mouseY, button);
		}
		return super.mouseReleased(mouseX, mouseY, button);
	}

	@Override
	public boolean mouseDragged(double mouseX, double mouseY, int button, double deltaX, double deltaY) {
		ChunkGrid.instance.onDragged(mouseX, mouseY, button);
		return super.mouseDragged(mouseX, mouseY, button, deltaX, deltaY);
	}

	private class NumberFieldWidget extends TextFieldWidget {
		private int lastValidValue;

		private NumberFieldWidget(TextRenderer textRenderer, int x, int y, int width, int height, Text text) {
			super(textRenderer, x, y, width, height, text);
		}

		private void setInitialValue(int value) {
			this.lastValidValue = value;
			this.setText(String.valueOf(value));
		}

		private int getValue() {
			return this.lastValidValue;
		}

		@Override
		public void setTextFieldFocused(boolean focused) {
			if (this.isFocused() && !focused) {
				Integer newValue = EssentialUtils.catchAsNull(() -> Integer.parseInt(this.getText()));
				if (newValue == null) {
					this.setText(String.valueOf(this.lastValidValue));
				}
				else if (this.lastValidValue != newValue) {
					this.lastValidValue = newValue;
					ChunkGrid.instance.setCentre(
						ChunkDebugScreen.this.xPositionBox.getValue(),
						ChunkDebugScreen.this.zPositionBox.getValue()
					);
				}
			}
			super.setTextFieldFocused(focused);
		}
	}
}
